"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@typescript-eslint/utils");
const utils_2 = require("@typescript-eslint/utils");
const no_unsafe_optional_property_assignment_1 = __importDefault(require("./no-unsafe-optional-property-assignment"));
// eslint-disable-next-line functional/prefer-immutable-types
const ruleTester = new utils_2.ESLintUtils.RuleTester({
    parserOptions: {
        sourceType: "module",
        project: "./tsconfig.tests.json",
    },
    parser: "@typescript-eslint/parser",
});
// eslint-disable-next-line functional/no-expression-statements
ruleTester.run("no-unsafe-optional-property-assignment", no_unsafe_optional_property_assignment_1.default, {
    valid: [
        {
            filename: "file.ts",
            code: `
        type Foo = { readonly foo: string };
        type Bar = { readonly foo: string; readonly bar: (() => unknown) | undefined };
        const thing = { foo: "foo", bar: "bar" };
        const foo: Foo = thing;
        const bar: Bar = { ...foo, bar: undefined };
      `,
        },
    ],
    invalid: [
        {
            filename: "file.ts",
            code: `
        type Foo = { readonly foo: string };
        type Bar = { readonly foo: string; readonly bar?: () => unknown };
        const thing = { foo: "foo", bar: "bar" };
        const foo: Foo = thing;
        const bar: Bar = foo;
      `,
            errors: [
                {
                    messageId: "errorStringVariableDeclaration",
                    type: utils_1.AST_NODE_TYPES.VariableDeclaration,
                },
            ],
        },
        {
            filename: "file.ts",
            code: `
        type Foo = { readonly foo: string };
        type Bar = Foo & { readonly bar?: () => unknown };
        const thing = { foo: "foo", bar: "bar" };
        const foo: Foo = thing;
        const bar: Bar = foo;
      `,
            errors: [
                {
                    messageId: "errorStringVariableDeclaration",
                    type: utils_1.AST_NODE_TYPES.VariableDeclaration,
                },
            ],
        },
    ],
});
